@@ -1,11 +1,13 @@
 import requests
 import urllib3
 import time
+import functools
 from log import log
 
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
 def enter_and_leave_function(func):
+    @functools.wraps(func)
     def wrapper(*args, **kwargs):
         if args:
             if kwargs:
@@ -137,11 +139,7 @@ class CloudPods:
             return None
 
     @enter_and_leave_function
-    def create_server_by_guest_image(self,guest_image_id,disk_image_id,arch, disk_size, disks, nets_list, vm_name, sku,count=1,reset_new_password="",hypervisor="kvm"):
-        if arch == "x86_64":
-            bios="BIOS"
-        else:
-            bios="UEFI"
+    def create_server_by_guest_image(self,guest_image_id,disk_image_id,arch, disk_size, disks, nets_list, vm_name, sku="",count=1,reset_new_password="",hypervisor="kvm",bios="bios",host_id=""):
         nets=[{"network": net} for net in nets_list]
         server = {
             "auto_start": True,
@@ -149,28 +147,47 @@ class CloudPods:
             "hypervisor": hypervisor,
             "disable_delete": False,
             "__count__": 1,
-            "disks": [{"disk_type": "sys", "index": 0, "backend": "local", "size": disk_size, "image_id": disk_image_id,
-                       "medium": "ssd"}],
-            "sku": sku,
-            "reset_password": False,
-            "guest_image_id": guest_image_id,
             "deploy_telegraf": True,
             "os_arch": arch,
             "nets": nets,
             "prefer_region": "default",
-            "bios": bios,
-            "schedtags": [{"id": "094cfe0f-7fbf-451f-879d-d44151539240", "strategy": "exclude"}],
+            "bios": bios
         }
+        if hypervisor == "kvm":
+            server["guest_image_id"]=guest_image_id
+            server["sku"] = sku
+            server["schedtags"] = [{"id": "094cfe0f-7fbf-451f-879d-d44151539240", "strategy": "exclude"}]
+            server["disks"] = [{"disk_type": "sys", "index": 0, "backend": "local", "size": disk_size, "image_id": disk_image_id,
+                       "medium": "ssd"}]
+            for i in range(len(disks)):
+                disk = dict({})
+                disk["disk_type"] = "data"
+                disk["index"] = i + 1
+                disk["backend"] = "local"
+                disk["size"] = int(disks[i]) * 1024
+                disk["medium"] = "ssd"
+                server["disks"].append(disk)
+        if hypervisor == "baremetal":
+            if not host_id:
+                log.error("Failed to create server: Error. err msg is host_id is empty")
+                return None
+            rs=self.get_host_spec(host_id)
+
+            server["disks"] = [
+                        {"size":40960,"image_id":guest_image_id},
+                        {"size":-1,"fs":"ext4","mountpoint":"/opt/cloud/workspace"}
+                      ]
+            server["vcpu_count"]=rs["host"]["cpu"]
+            server["vmem_size"]=rs["host"]["mem"]
+            server["baremetal_disk_configs"]=[
+                        {"conf":"none","driver":"PCIE","count":1,"range":[0],"adapter":0,"type":"ssd"}
+                      ]
         if reset_new_password:
             server["password"]=reset_new_password
-        for i in range(len(disks)):
-            disk = dict({})
-            disk["disk_type"] = "data"
-            disk["index"] = i + 1
-            disk["backend"] = "local"
-            disk["size"] = int(disks[i]) * 1024
-            disk["medium"] = "ssd"
-            server["disks"].append(disk)
+            server["reset_password"] = True
+        else:
+            server["reset_password"] = False
+
         res = self.__create_server(server,count)
         server_ids=[]
         if "server" in res.keys() and "id" in res["server"].keys():
@@ -185,17 +202,14 @@ class CloudPods:
 
     @enter_and_leave_function
     def get_server_detail(self, server_id):
-        res_data={}
         try:
             get_all_snapshots = f"/servers/{server_id}"
             url = self.__compute_url + get_all_snapshots
-            res = self.__session.get(url=url, verify=False, timeout=600)
-            res_data = res.json()
-            log.info(f"Success to request /servers/{server_id}: OK. response data is {res_data}")
+            rs = self.__session.get(url=url, verify=False, timeout=600)
+            rs_data = rs.json()
+            return rs_data
         except Exception as e:
             log.error(f"Failed to request /servers/{server_id}: Error.\rerr msg is {str(e)}",exc_info=True)
-        finally:
-            return res_data
 
     @enter_and_leave_function
     def get_server_ip(self, server_id):
@@ -248,6 +262,80 @@ class CloudPods:
                 log.error(f"Failed to get server_detail: Error. server_detail is: {server_detail}")
                 return False
 
+    @enter_and_leave_function
+    def host_has_baremetal_server(self, host_id):
+        try:
+            url=self.__compute_url+"/servers"
+            payload = {
+                "scope": "system",
+                "hypervisor": "baremetal",
+                "host": host_id,
+                "details": True,
+                "baremetal": True,
+                "with_meta": True,
+                "summary_stats": True
+            }
+            rs = self.__session.get(url=url, params=payload, verify=False, timeout=600)
+            rs_data = rs.json()
+            return bool(rs_data["servers"])
+        except Exception as e:
+            log.error(f"Failed to request /servers: Error.\rerr msg is {str(e)}",exc_info=True)
+            return False
+
+    @enter_and_leave_function
+    def get_host_spec(self,host_id):
+        try:
+            url=self.__compute_url+"/hosts/"+host_id+"/spec"
+            payload = {
+                "scope": "system",
+                "details": True,
+                "baremetal": True,
+                "with_meta": True,
+                "summary_stats": True
+            }
+            rs = self.__session.get(url=url, params=payload, verify=False, timeout=600)
+            rs_data = rs.json()
+            return rs_data
+        except Exception as e:
+            log.error(f"Failed to request /hosts/{host_id}/spec: Error.\rerr msg is {str(e)}",exc_info=True)
+            return None
+
+    @enter_and_leave_function
+    def get_baremetal_hosts(self):
+        try:
+            get_all_baremetal_hosts = f"/hosts"
+            url = self.__compute_url + get_all_baremetal_hosts
+            payload = {
+                "scope": "system",
+                "details": True,
+                "baremetal": True,
+                "with_meta": True,
+                "summary_stats":  True
+            }
+            rs = self.__session.get(url=url, params=payload, verify=False, timeout=600)
+            rs_data = rs.json()
+            return rs_data
+        except Exception as e:
+            log.error(f"Failed to request /hosts: Error.\rerr msg is {str(e)}",exc_info=True)
+
+    @enter_and_leave_function
+    def get_baremetal_servers(self):
+        try:
+            get_all_baremetal_servers = f"/servers"
+            url = self.__compute_url + get_all_baremetal_servers
+            payload = {
+                "scope": "system",
+                "hypervisor": "baremetal",
+                "details": True,
+                "with_meta": True,
+                "summary_stats": True
+            }
+            rs = self.__session.get(url=url, params=payload, verify=False, timeout=600)
+            rs_data = rs.json()
+            return rs_data
+        except Exception as e:
+            log.error(f"Failed to request /servers: Error.\rerr msg is {str(e)}", exc_info=True)
+
     @enter_and_leave_function
     def delete_server(self, server_id):
         try:
@@ -267,6 +355,28 @@ class CloudPods:
 
 
 if __name__ == '__main__':
-    cloudpods = CloudPods('https://10.30.18.1:30500/v3','admin','jSj@2008')
-    server_ip = cloudpods.get_server_ip("e3b76ec7-c4f9-4065-8ce2-9d833b51368c")
-    print(server_ip)
\ No newline at end of file
+    # cloudpods = CloudPods('https://10.30.18.1:30500/v3','admin','jSj@2008')
+    # server_ip = cloudpods.get_server_ip("e3b76ec7-c4f9-4065-8ce2-9d833b51368c")
+    # print(server_ip)
+    # cloudpods = CloudPods('https://10.240.30.110:30500/v3', 'admin', 'jSj@2008')
+    # rs=cloudpods.get_baremetal_hosts()
+    # print(rs)
+    # print(len(rs["hosts"]))
+    # rs=cloudpods.get_baremetal_servers()
+    # print(rs)
+    # print(len(rs["servers"]))
+    # rs=cloudpods.delete_server("8c925c8e-752f-4358-8c3e-a80847732d91")
+    # print(rs)
+    # rs=cloudpods.host_has_baremetal_server("fa09da4e-b6d7-4a07-855d-d690abea66a9")
+    # print(rs)
+    # rs=cloudpods.host_has_baremetal_server("701f77bf-8cac-4b20-8787-41dc3ddc3f77")
+    # print(rs)
+    # rs=cloudpods.get_host_spec("fa09da4e-b6d7-4a07-855d-d690abea66a9")
+    # print(rs)
+    # rs=cloudpods.create_server_by_guest_image("8572ed8a-6ffe-451a-80ac-c04da95e777f","1fdfbc15-e319-4f59-8e2b-cc45bc3517cb",
+    #                                           "x86_64",20480,[20],["265005a2-b3a4-42e3-8154-471bdd694487"],"demo2",hypervisor="baremetal",bios="UEFI",
+    #                                           host_id="8bb7f77c-d3bf-432b-85e8-8094ba0e8cd5"
+    #                                           )
+    # print(rs)
+    pass
+
